@@ -1,36 +1,9 @@
-#
-#  Copyright (c) 1997-2002 The Protein Laboratory, University of Copenhagen
-#  All rights reserved.
-#
-#  Redistribution and use in source and binary forms, with or without
-#  modification, are permitted provided that the following conditions
-#  are met:
-#  1. Redistributions of source code must retain the above copyright
-#     notice, this list of conditions and the following disclaimer.
-#  2. Redistributions in binary form must reproduce the above copyright
-#     notice, this list of conditions and the following disclaimer in the
-#     documentation and/or other materials provided with the distribution.
-#
-#  THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
-#  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
-#  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
-#  ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
-#  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
-#  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
-#  OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
-#  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
-#  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
-#  OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
-#  SUCH DAMAGE.
-#
-#  Created by Dmitry Karasik <dk@plab.ku.dk>
-#
-#  $Id$
-
 # contains:
 #   CheckList
 
+package Prima::ExtLists;
 use strict;
+use warnings;
 use Prima::Const;
 use Prima::Classes;
 use Prima::Lists;
@@ -40,13 +13,16 @@ package Prima::CheckList;
 use vars qw(@ISA);
 @ISA = qw(Prima::ListBox);
 
-my @images = (
-	Prima::StdBitmap::image(sbmp::CheckBoxUnchecked),
-	Prima::StdBitmap::image(sbmp::CheckBoxChecked),
-);
+my (@images, @imgSize);
 
-my @imgSize = (0,0);
-@imgSize = $images[0]-> size if $images[0];
+Prima::Application::add_startup_notification( sub {
+	@images = (
+		Prima::StdBitmap::image(sbmp::CheckBoxUnchecked),
+		Prima::StdBitmap::image(sbmp::CheckBoxChecked),
+	);
+	@imgSize = (0,0);
+	@imgSize = $images[0]-> size if $images[0];
+});
 
 sub profile_default
 {
@@ -107,8 +83,8 @@ sub draw_text_items
 		next if $self-> {widths}-> [$i] + $self-> {offset} + $x + 1 < $clipRect-> [0];
 		$canvas-> text_out_bidi( $self-> {items}-> [$i], $x + 2 + $imgSize[0],
 			$y + $textShift - $j * $self-> {itemHeight} + 1);
-		$canvas-> put_image( $x + 1, 
-			$y + int(( $self-> {itemHeight} - $imgSize[1]) / 2) - 
+		$canvas-> put_image( $x + 1,
+			$y + int(( $self-> {itemHeight} - $imgSize[1]) / 2) -
 				$j * $self-> {itemHeight} + 1,
 			$images[ vec($self-> {vector}, $i, 1)],
 		);
@@ -173,7 +149,7 @@ sub button
 	my $current = vec( $self-> {vector}, $index, 1);
 	return $current unless defined $state;
 	$state = ( $state < 0) ? !$current : ( $state ? 1 : 0);
-	return $current if $current == $state; 
+	return $current if $current == $state;
 	vec( $self-> {vector}, $index, 1) = $state;
 	$self-> notify(q(Change), $index, $state);
 	$self-> redraw_items( $index);
@@ -187,8 +163,6 @@ sub button
 
 1;
 
-__DATA__
-
 =pod
 
 =head1 NAME
@@ -197,7 +171,7 @@ Prima::ExtLists - extended functionality for list boxes
 
 =head1 SYNOPSIS
 
-	use Prima::ExtLists;
+	use Prima qw(ExtLists Application);
 
 	my $vec = '';
 	vec( $vec, 0, 8) = 0x55;
@@ -205,8 +179,13 @@ Prima::ExtLists - extended functionality for list boxes
 		items  => [1..10],
 		vector => $vec,
 	);
+	run Prima;
 
-=head1 DESCRIPTION 
+=for podview <img src="extlist.gif" cut=1>
+
+=for html <p><img src="https://raw.githubusercontent.com/dk/Prima/master/pod/Prima/extlist.gif">
+
+=head1 DESCRIPTION
 
 The module is intended to be a collection of list boxes with
 particular enhancements. Currently the only package is contained
@@ -232,7 +211,7 @@ Returns the new state of the button.
 =item vector VEC
 
 VEC is a vector scalar, where each bit corresponds to the check state
-of each list box item. 
+of each list box item.
 
 See also: L<perlfunc/vec>.
 
@@ -260,5 +239,5 @@ Dmitry Karasik, E<lt>dmitry@karasik.eu.orgE<gt>.
 
 L<Prima>, L<Prima::Lists>, F<examples/extlist.pl>
 
-=cut 
+=cut
 
