@@ -1,6 +1,6 @@
 #  Created by:
 #     Anton Berezin  <tobez@tobez.org>
-#     Dmitry Karasik <dk@plab.ku.dk> 
+#     Dmitry Karasik <dk@plab.ku.dk>
 #     Vadim Belman   <voland@plab.ku.dk>
 
 package Prima::Gencls;
@@ -197,7 +197,7 @@ sub init_variables
 
 	%typedefs = ();
 
-	%xsConv = (             
+	%xsConv = (
 		# declare   access      set          unused   unused      extra2sv POPx     newXXxx extrasv2
 		# 0         1            2           3        4           5        6        7       8
 		'int'     => ['int',    'SvIV',      'sv_setiv',  '',      '(IV)',     '',    'POPi',    'SViv', ''     ],
@@ -299,7 +299,7 @@ sub getObjName
 	$tok = gettok;
 	for ( split( '::', $tok))
 	{
-		error "Expecting object identifier but found ``$_''" 
+		error "Expecting object identifier but found ``$_''"
 			unless $_ =~ /^\w+$/;
 	}
 	return $tok;
@@ -521,7 +521,7 @@ sub parse_struc
 		if ( $strucId eq "%")
 		{
 			my $def;
-			if 
+			if
 			( $type eq 'Handle') { $def = '( Handle) C_POINTER_UNDEF'; } elsif
 			( $type eq 'SV*')    { $def = '( SV*) C_POINTER_UNDEF'; } elsif
 			( $type eq 'int')    { $def = 'C_NUMERIC_UNDEF'; } elsif
@@ -530,7 +530,11 @@ sub parse_struc
 			( $type eq 'char*')  { $def = 'C_STRING_UNDEF'; } elsif
 			( $type eq 'string') { $def = 'C_STRING_UNDEF'; };
 			$tok = gettok;
-			if ( $tok eq '=') {
+			if ( $tok eq 'with' ) {
+				expect('undef');
+				expect(';');
+				$def = "undef:$def";
+			} elsif ( $tok eq '=') {
 				$def = gettok;
 				expect(';');
 			} else {
@@ -713,10 +717,10 @@ sub preload_method
 			if ( $tok eq '...') {
 				$ellipsis  = 1;
 				$hasEllipsis++;
-				error "Property can not contain ellipsis into parameter list" 
+				error "Property can not contain ellipsis into parameter list"
 					if $asProperty;
 			} else {
-				error "Invalid parameter $parmNo description" 
+				error "Invalid parameter $parmNo description"
 					unless defined $gp[0];
 			}
 			my $hasEqState = 0;
@@ -728,7 +732,7 @@ sub preload_method
 				error "Indirect function and array parameters are unsupported. Error"
 					if /^[\(\)\[\]\&]$/;
 				error "Unexpected semicolon" if /^[\;]$/;
-				error "End of default parameter description expected but found $_" 
+				error "End of default parameter description expected but found $_"
 					if $hasEqState > 1;
 				if ( $hasEqState == 1)
 				{
@@ -737,9 +741,9 @@ sub preload_method
 				}
 				# default parameter add
 				if ( $_ eq "=") {
-					error "Invalid default parameter $parmNo description" 
+					error "Invalid default parameter $parmNo description"
 						if $hasEqState;
-					error "Property can not contain default parameters" 
+					error "Property can not contain default parameters"
 						if $asProperty;
 					$hasEqState = 1;
 				}
@@ -855,27 +859,27 @@ sub parse_parms
 		{
 			my $keyType = $lxmz[ $j];
 			my $orgType = $keyType;
-			if ( 
-				exists( $structs{ $keyType}) || 
-				exists( $arrays{ $keyType}) || 
+			if (
+				exists( $structs{ $keyType}) ||
+				exists( $arrays{ $keyType}) ||
 				( $keyType = $mapTypes{ $lxmz[ $j]})
 			) {
 				last if exists( $mapPointers{ $lxmz[ $j]}) && ( $lxmz[$j+1] ne "*");
 				if ($lxmz[ $j+1] eq "*") {
 					if ( exists( $mapPointers{ $lxmz[ $j]})) {
-						if ( $lxmz[ $j+2] eq "*") { 
-							last 
+						if ( $lxmz[ $j+2] eq "*") {
+							last
 						} else {
 							$orgType = $keyType = $mapPointers{ $lxmz[ $j]}
 						}
 					} else {
-						if ( 
-							exists( $structs{ $keyType}) || 
+						if (
+							exists( $structs{ $keyType}) ||
 							exists( $arrays{ $keyType})
 						) {
 							$orgType = $keyType = "*".$lxmz[$j];
-						} else { 
-							last 
+						} else {
+							last
 						};
 					}
 				}
@@ -944,9 +948,9 @@ sub load_single_part
 			}
 			expect ")";
 			$tok = gettok;
-			error "Cyclic inheritance detected.  Watch out ``$superClass''" 
+			error "Cyclic inheritance detected.  Watch out ``$superClass''"
 				if $definedClasses{ $superClass};
-				
+
 			do {
 				$level++;
 				if ( $suppress & (!$sayparent || $level==1)) {
@@ -957,9 +961,9 @@ sub load_single_part
 				putback( $tok);
 				my ( $superFile, $inhDyna) = find_file( "$superClass.cls");
 				$tok = gettok;
-				error "Cannot find file corresponding to ''$superClass''" 
+				error "Cannot find file corresponding to ''$superClass''"
 					unless defined $superFile;
-				error "Static object cannot inherit dynamic module $superClass" 
+				error "Static object cannot inherit dynamic module $superClass"
 					if !$genDyna && $inhDyna;
 				parse_file( $superFile) unless $sayparent;
 				$level--;
@@ -981,10 +985,10 @@ sub load_single_part
 				unless ($level) {
 					if ( parse_parms) {
 						push (@portableMethods, "${id}${acc}");
-						push (@newMethods, "${id}${acc}") 
+						push (@newMethods, "${id}${acc}")
 							unless $inherit;
 					}  else {
-						print "Warning: method $id has not been published\n" 
+						print "Warning: method $id has not been published\n"
 							if $warnings;
 					}
 				};
@@ -995,7 +999,7 @@ sub load_single_part
 					if ( parse_parms) {
 						push (@newMethods, "${id}${acc}") unless $inherit;
 					}  else {
-						print "Warning: method $id has not been published\n" 
+						print "Warning: method $id has not been published\n"
 							if $warnings;
 					}
 				};
@@ -1003,7 +1007,7 @@ sub load_single_part
 				preload_method( 1, 0);
 				store_method;
 				unless ($level) {
-					error "Invalid types in import function $id declaration" 
+					error "Invalid types in import function $id declaration"
 						unless parse_parms;
 					push (@portableImports, "${id}${acc}");
 					push (@newMethods, "${id}${acc}") unless $inherit;
@@ -1017,7 +1021,7 @@ sub load_single_part
 						push (@portableMethods, "${id}${acc}");
 						push (@newMethods, "${id}${acc}") unless $inherit;
 					} else {
-						print "Warning: static $id has not been published\n" 
+						print "Warning: static $id has not been published\n"
 							if $warnings;
 					}
 				};
@@ -1051,14 +1055,14 @@ sub load_single_part
 						push (@portableMethods, "$id$acc");
 						push (@newMethods, "$id$acc") unless $inherit;
 					} else {
-						print "Warning: property $id has not been published\n" 
+						print "Warning: property $id has not been published\n"
 							if $warnings;
 					}
 				}
 				$properties{$id} = $proptype;
 			} else {
 			# variable workaround
-				error "Expecting identifier but found ``$tok''" 
+				error "Expecting identifier but found ``$tok''"
 					unless $tok =~ /^\w+$/;
 				$acc = $tok;
 				until (($tok = getidsym("*;[]()+")) eq ";")
@@ -1083,7 +1087,7 @@ sub load_single_part
 					push (@portableMethods, "${id}${acc}");
 					push (@newMethods, "${id}${acc}") unless $inherit;
 				} else {
-					print "Warning: function $id has not been published\n" 
+					print "Warning: function $id has not been published\n"
 						if $warnings;
 				}
 			};
@@ -1237,13 +1241,13 @@ sub out_method_profile
 		my $useHandle = !exists( $staticMethods{ $id});
 		my $tok = $allMethodsBodies[$allMethods{$id}];
 		my $exStr = eval( "\"$substring\"");
-		
+
 		$tok =~ s/$id(?=\()/$exStr/;      # incorporating class name
 		$tok =~ s/;([^;]*)$/$1/;          # adding ; to the end
-		
+
 		print( HEADER "$tok;\n\n"), next if $publicMethods{$id} && $full;
 		my $castedResult = $parms[0];
-		$castedResult = exists $typedefs{$castedResult} && $typedefs{$castedResult}{cast} 
+		$castedResult = exists $typedefs{$castedResult} && $typedefs{$castedResult}{cast}
 			? " ( $castedResult)" :
 			'';
 		my $subId = $full ? ( $imports ? "\"$ownOClass\:\:$id\"" : "\"$id\"") : 'subName';
@@ -1308,6 +1312,7 @@ sub out_method_profile
 			print HEADER "\t$result $eptr $incRes;\n\n";
 		}
 		}
+		print HEADER "\t(void)$incCount;\n";
 		print HEADER "\tENTER;\n";
 		print HEADER "\tSAVETMPS;\n";
 		print HEADER "\tPUSHMARK( sp);\n\n";
@@ -1316,7 +1321,7 @@ sub out_method_profile
 		# writing other parameters
 		for ( my $j = 0; $j < scalar @parm_ids; $j++) {
 			$j = 1 if $j == 0 && $property;
-			print HEADER " \n\tif ( !set) goto CALL_POINT;\n" 
+			print HEADER " \n\tif ( !set) goto CALL_POINT;\n"
 				if $property && $j == $#parm_ids;
 
 			my $lVar = $parm_ids[ $j];
@@ -1324,8 +1329,8 @@ sub out_method_profile
 			my $ptr  = $type =~ /^\*/ ? "*" : "";
 			$type =~ s[^\*][];
 
-			my $hashStruc = 
-				exists($structs{ $type}) && 
+			my $hashStruc =
+				exists($structs{ $type}) &&
 				defined ${$structs{ $type}[2]}{hash};
 			if ( $hashStruc) {
 			# take down the reference in this case
@@ -1354,12 +1359,12 @@ sub out_method_profile
 				my $lCount = $arrays{ $type}[0];
 				my $mc = mortal( $lType);
 				my $adx = ( $lCount > 1) ? "\t" : "";
-				print HEADER "\t{\n\t\tint $incCount;\n" 
+				print HEADER "\t{\n\t\tint $incCount;\n"
 					if $adx;
-				print HEADER "\t\tfor ( $incCount = 0; $incCount < $lCount; ${incCount}++)\n" 
+				print HEADER "\t\tfor ( $incCount = 0; $incCount < $lCount; ${incCount}++)\n"
 					if $adx;
 				print HEADER "$adx\tXPUSHs( $mc( ", type2sv( $lType, "$lVar\[$incCount\]"), "));\n";
-				print HEADER "\t}\n" 
+				print HEADER "\t}\n"
 					if $adx;
 				# scalars
 			} else {
@@ -1377,13 +1382,13 @@ sub out_method_profile
 		print HEADER "CALL_POINT:\n" if $property;
 		print HEADER "\n\tPUTBACK;\n\n";
 		if ( exists( $structs{ $resSub}) && ${$structs{ $resSub}[2]}{hash}) {
-		$retType = "G_SCALAR";
+			$retType = "G_SCALAR";
 		} elsif ( exists( $structs{ $resSub}) || exists( $arrays{ $resSub}) || $hvName) {
-		$retType = "G_ARRAY";
+			$retType = "G_ARRAY";
 		} elsif ( $resSub eq "void") {
-		$retType = "G_DISCARD";
+			$retType = "G_DISCARD";
 		} else {
-		$retType = "G_SCALAR";
+			$retType = "G_SCALAR";
 		};
 
 		$retType = "set ? G_DISCARD : $retType" if $property;
@@ -1414,20 +1419,20 @@ sub out_method_profile
 			) : $arrays{ $resSub}[0];
 			my $resVar = $lpr.$eptr.$incRes.$rpr;
 			print HEADER "\tSPAGAIN;\n";
-			print HEADER "\t$incCount = pop_hv_for_REDEFINED( sp, $incCount, $hvName, $popParams);\n" 
+			print HEADER "\t$incCount = pop_hv_for_REDEFINED( sp, $incCount, $hvName, $popParams);\n"
 				if $hvName;
-			print HEADER "\tif ( set) {\n\t\tFREETMPS;\n\t\tLEAVE;\n\t\treturn $incRes;\n\t}\n\n" 
+			print HEADER "\tif ( set) {\n\t\tFREETMPS;\n\t\tLEAVE;\n\t\treturn $incRes;\n\t}\n\n"
 				if $property;
 			print HEADER "\tif ( $incCount != $lParams) croak( \"Sub result corrupted\");\n\n";
 			if ( exists $structs{ $resSub}) {
 				# struct
 				if ( defined ${$structs{ $resSub}[2]}{hash}) {
-					print HEADER "\t$incRes = \& $castedResult ${result}_buffer;\n" 
+					print HEADER "\t$incRes = \& $castedResult ${result}_buffer;\n"
 						if $eptr;
 					my $xref = $eptr ? '' : '&';
 					print HEADER "\tSvHV_$resSub( POPs, $castedResult $xref$incRes, $subId);\n";
 				} else {
-					print HEADER "\t$incRes = $castedResult ${result}_buffer;\n" 
+					print HEADER "\t$incRes = $castedResult ${result}_buffer;\n"
 						if $eptr;
 					for ( my $j = 0; $j < $lParams; $j++) {
 						my $lType = @{ $structs{ $resSub}[ 0]}[ $lParams - $j - 1];
@@ -1441,13 +1446,13 @@ sub out_method_profile
 				print HEADER "\t$incRes =$castedResult \&${result}_buffer;\n" if $eptr;
 				my $lType   = $arrays{ $resSub}[1];
 				my $adx = ( $lParams > 1) ? "\t" : "";
-				print HEADER "\t{\n\tint $incCount;\n" 
+				print HEADER "\t{\n\tint $incCount;\n"
 					if $adx;
-				print HEADER "\tfor ( $incCount = 0; $incCount < $lParams; ${incCount}++)\n" 
+				print HEADER "\tfor ( $incCount = 0; $incCount < $lParams; ${incCount}++)\n"
 					if $adx;
-				print HEADER "$adx\t", 
-					cwrite( $lType, sv2type_pop( $lType), 
-					"$resVar\[$incCount\]"), 
+				print HEADER "$adx\t",
+					cwrite( $lType, sv2type_pop( $lType),
+					"$resVar\[$incCount\]"),
 					"\n";
 				print HEADER "\t}\n" if $adx;
 			}
@@ -1467,9 +1472,9 @@ sub out_method_profile
 		} else {
 		# 1 param to return
 			print HEADER "\tSPAGAIN;\n";
-			print HEADER "\t$incCount = pop_hv_for_REDEFINED( sp, $incCount, $hvName, 1);\n" 
+			print HEADER "\t$incCount = pop_hv_for_REDEFINED( sp, $incCount, $hvName, 1);\n"
 				if $hvName;
-			print HEADER "\tif ( set) {\n\t\tFREETMPS;\n\t\tLEAVE;\n\t\treturn ( $resSub$eptr) 0;\n\t}\n\n" 
+			print HEADER "\tif ( set) {\n\t\tFREETMPS;\n\t\tLEAVE;\n\t\treturn ( $resSub$eptr) 0;\n\t}\n\n"
 				if $property;
 			print HEADER "\tif ( $incCount != 1) croak( \"Something really bad happened!\");\n\n";
 			if ( $resSub eq 'Handle') {
@@ -1520,8 +1525,8 @@ sub out_FROMPERL_methods
 				$thunks{$templates_xs{$id}} = 1;
 				print HEADER "extern void $templates_xs{$id}( XS_STARTPARAMS, char* subName, void* func);\n\n";
 			}
-			my $func = ( exists $pipeMethods{ $id}) ? 
-				$pipeMethods{$id} : 
+			my $func = ( exists $pipeMethods{ $id}) ?
+				$pipeMethods{$id} :
 				"${ownCType}_$id";
 			print HEADER <<LABEL;
 XS( ${ownCType}_${id}_FROMPERL) {
@@ -1634,7 +1639,7 @@ LABEL
 		print HEADER "\t{\n\t\t";
 		my $structCount = 0;
 		my $stn = $useHandle ? 1 : 0;
-		unless ($resSub eq "void") { 
+		unless ($resSub eq "void") {
 			print HEADER "$result $eptr $incRes;\n\t\t"
 		};
 		# generating struct local vars
@@ -1646,22 +1651,22 @@ LABEL
 			my $lVar = $_; $lVar =~ s[^\*][];
 			my ( $lp, $rp) = $ptr ? ('(',')') : ('','');
 			if ( exists $structs{$lVar} && defined ${$structs{$lVar}[2]}{hash}) {
-				$reuseStructVar = 1, last 
+				$reuseStructVar = 1, last
 					if $property && $idparm == $#parms && ( $ptr eq "");
 				print HEADER "$lVar $incRes$structCount;\n\t\t";
 				$structCount++;
 			} elsif ( exists $arrays{$lVar} || exists $structs{$lVar}) {
-				$reuseStructVar = 1, last 
+				$reuseStructVar = 1, last
 					if $property && $idparm == $#parms && ( $ptr eq "");
 				print HEADER "$lVar $incRes$structCount;\n\t\t";
 				$structCount++;
-			} else { 
-				$stn++ 
+			} else {
+				$stn++
 			};
 			$idparm++;
 		}
 		my $subId = $full ? "\"${ownClass}_$id\"" : 'subName';
-		print HEADER "HV *hv = parse_hv( ax, sp, items, mark, $nParam - 1, $subId);\n\t\t" 
+		print HEADER "HV *hv = parse_hv( ax, sp, items, mark, $nParam - 1, $subId);\n\t\t"
 			if $useHV;
 		# initializing strings and additional parameters, if any
 		$stn = $useHandle ? 1 : 0;
@@ -1709,8 +1714,8 @@ LABEL
 				my $str;
 				$paramAuxSet .= "{\n\t\t\tint $incCount;\n\t\t\t";
 				$paramAuxSet .= "for ( $incCount = 0; $incCount < $arrays{$lVar}[0]; $incCount++)\n\t\t\t";
-				if ( $lName eq 'SV*') { 
-					$str = "$incRes$structCount\[$incCount\] = ST( $stn + $incCount)" 
+				if ( $lName eq 'SV*') {
+					$str = "$incRes$structCount\[$incCount\] = ST( $stn + $incCount)"
 				} elsif ( $lName eq 'string') {
 					$str = "strncpy( $incRes$structCount\[$incCount\], ( char*) SvPV_nolen( ST( $stn + $incCount)), 255);$incRes$structCount\[$incCount\]\[255\]=0"
 				} else {
@@ -1719,8 +1724,8 @@ LABEL
 				$paramAuxSet .= "\t$str;\n\t\t}\n\t\t";
 				$stn += $arrays{$lVar}[0];
 				$structCount++;
-			} else { 
-				$stn++ 
+			} else {
+				$stn++
 			};
 			$idparm++;
 		}
@@ -1736,14 +1741,14 @@ LABEL
 		}
 		print HEADER "$incRes = " unless $resSub eq "void";
 		if ( $full) {
-			print HEADER ( exists $pipeMethods{ $id}) ? 
-				$pipeMethods{$id} : 
+			print HEADER ( exists $pipeMethods{ $id}) ?
+				$pipeMethods{$id} :
 				"${ownCType}_$id";
 		} else {
 			my @parmList = @parms;
 			unshift( @parmList, 'Bool') if $property;
 			unshift( @parmList, 'Handle') if $useHandle;
-			for ( @parmList) { 
+			for ( @parmList) {
 				s/^\*(.*)/$1\*/;
 			}  # since "*Struc" if way to know it's user ptr, map it back
 			my $parmz = join( ', ', @parmList);
@@ -1771,14 +1776,14 @@ LABEL
 			$structCount = '' if $property && $reuseStructVar && $idparm == $#parms;
 			print HEADER ",\n" if $stn > 0;
 			print HEADER "\t\t\t";
-			print HEADER "( $ifpropset) ? ( " if 
+			print HEADER "( $ifpropset) ? ( " if
 				$property && !$reuseStructVar && $idparm == $#parms;
 			my $defPropParm;
 			if ( exists $structs{$lVar} || exists $arrays{$lVar})
 			{
 				if ( exists $structs{$lVar}) {
-					$stn += defined ${$structs{$lVar}[2]}{hash} ? 
-						1 : 
+					$stn += defined ${$structs{$lVar}[2]}{hash} ?
+						1 :
 						scalar @{ $structs{ $lVar}[ 0]};
 				} else {
 					$stn += $arrays{$lVar}[0];
@@ -1797,15 +1802,15 @@ LABEL
 				$defPropParm = "($lVar)0";
 				$stn++;
 			}
-			print HEADER ") : $defPropParm" if 
+			print HEADER ") : $defPropParm" if
 				$property && !$reuseStructVar && $idparm == $#parms;
 			$idparm++;
 		}
 		print HEADER "\n";
 		print HEADER "\t\t);\n";
-		print HEADER "\t\tSPAGAIN;\n\t\tSP -= items;\n" 
+		print HEADER "\t\tSPAGAIN;\n\t\tSP -= items;\n"
 			if (!( $resSub eq "void") || $useHV);
-		print HEADER "\t\tif ( $ifpropset) {\n\t\t\tXSRETURN_EMPTY;\n\t\t\treturn;\n\t\t}\n" 
+		print HEADER "\t\tif ( $ifpropset) {\n\t\t\tXSRETURN_EMPTY;\n\t\t\treturn;\n\t\t}\n"
 			if $property;
 
 		my $pphv = 0;
@@ -1835,9 +1840,9 @@ LABEL
 			$lType = $mapTypes{$lType}||$lType;
 			print HEADER "\t\tEXTEND(sp, $rParms);\n";
 			my $adx = ( $rParms > 1) ? "\t\t" : "";
-			print HEADER "\t\t{\n\t\t\tint $incCount;\n" 
+			print HEADER "\t\t{\n\t\t\tint $incCount;\n"
 				if $adx;
-			print HEADER "\t\t\tfor ( $incCount = 0; $incCount < $rParms; ${incCount}++)\n" 
+			print HEADER "\t\t\tfor ( $incCount = 0; $incCount < $rParms; ${incCount}++)\n"
 				if $adx;
 			my $mc = mortal( $lType);
 			my $inter = type2sv( $lType, "$lpr$eptr$incRes$rpr\[$incCount\]");
@@ -1861,7 +1866,7 @@ LABEL
 				print HEADER "\t\tXPUSHs( sv_2mortal( new$xsConv{$resSub}[7]( $incRes$xsConv{$resSub}[5])));\n";
 			}
 		};
-		print HEADER "\t\tpush_hv( ax, sp, items, mark, $pphv, hv);\n\t\treturn;\n" 
+		print HEADER "\t\tpush_hv( ax, sp, items, mark, $pphv, hv);\n\t\treturn;\n"
 			if $useHV;
 		print HEADER "\t}\n";
 		print HEADER $pphv ? "\tPUTBACK;\n\treturn;\n" : "\tXSRETURN_EMPTY;\n";
@@ -1937,7 +1942,7 @@ LABEL
 print HEADER "#include \"$baseFile.h\"\n" if $baseClass;
 {
 	my %toInclude = ();
-	for ( sort { 
+	for ( sort {
 		$structs{$a}-> [PROPS]-> {order} <=> $structs{$b}-> [PROPS]-> {order}
 	} keys %structs) {
 		my $f = ${$structs{$_}[2]}{incl};
@@ -1948,7 +1953,7 @@ print HEADER "#include \"$baseFile.h\"\n" if $baseClass;
 		my $f = ${$arrays{$_}[2]}{incl};
 		$toInclude{$f}=1 if $f;
 	}
-	
+
 	for ( keys %defines) {
 		my $f = $defines{$_}{incl};
 		$toInclude{$f}=1 if $f;
@@ -1977,8 +1982,10 @@ for ( sort { $structs{$a}-> [PROPS]-> {order} <=> $structs{$b}-> [PROPS]-> {orde
 	if ( ${$structs{$_}[PROPS]}{genh})
 	{
 		my @types = @{$structs{$_}-> [TYPES]};
-		my @ids = @{$structs{$_}-> [IDS]};
+		my @ids   = @{$structs{$_}-> [IDS]};
+		my @def   = @{$structs{$_}-> [DEFS]};
 		print HEADER "typedef struct _$_ {\n";
+		my ($maxw_undefs, @undefs) = (0);
 		for ( my $j = 0; $j < @types; $j++) {
 			if ( ref $types[$j]) {
 				print HEADER "\t$types[$j]->[PROPS]->{name} $ids[$j];\n";
@@ -1987,6 +1994,16 @@ for ( sort { $structs{$a}-> [PROPS]-> {order} <=> $structs{$b}-> [PROPS]-> {orde
 			} else {
 				print HEADER "\t$types[$j] $ids[$j];\n";
 			}
+
+			if (($def[$j] // '') =~ /^undef:/) {
+				push @undefs, $ids[$j];
+				$maxw_undefs = length $ids[$j] if length($ids[$j]) > $maxw_undefs;
+			}
+		}
+		if ( @undefs ) {
+			print HEADER "\tstruct {\n";
+			printf HEADER "\t\tunsigned %\-${maxw_undefs}s : 1;\n", $_ for @undefs;
+			print HEADER "\t} undef;\n";
 		}
 		print HEADER "} $_, *P$_;\n\n";
 		if ( ${$structs{$_}[PROPS]}{hash})
@@ -2081,7 +2098,7 @@ if ( $genPackage)
 		my $id = $allMethods[$i];
 		my $body = $allMethodsBodies[$i];
 		$body =~ s{$id\(}{${ownCType}_$id\(};
-		print HEADER "extern $body\n" 
+		print HEADER "extern $body\n"
 			if ( $allMethodsHosts{$id} eq $ownOClass) && !exists( $pipeMethods{$id});
 	}
 }
@@ -2096,8 +2113,8 @@ my %propBody = map {
 
 for ( keys %properties) {
 	my $type = $properties{$_};
-	my $def  = ( exists $structs{ $type} || exists( $arrays{ $type})) ? 
-		"${type}_buffer" : 
+	my $def  = ( exists $structs{ $type} || exists( $arrays{ $type})) ?
+		"${type}_buffer" :
 		"($type)0";
 	my @lval;
 	my @rval;
@@ -2189,9 +2206,13 @@ SD
 	}
 CONTAINED_STRUCTURE
 				} else {
-					$inter = "( $incSV = hv_fetch( $incHV, \"$lName\", $lNameLen, 0)) ? \n\t".
-						sv2type( $lType, "*$incSV")." : $def";
-					print HEADER "\t", cwrite( $lType, $inter, "strucRef-> $lName"), "\n";
+					print HEADER "\t$incSV = hv_fetch( $incHV, \"$lName\", $lNameLen, 0);\n";
+					if ($def =~ /^undef:(.*)$/) {
+						print HEADER "\tstrucRef-> undef.$lName = ($incSV == NULL);\n";
+						$def = $1;
+					}
+					$inter = "$incSV ? " . sv2type( $lType, "*$incSV") . " : $def";
+					print HEADER "\t", cwrite( $lType, $inter, "strucRef-> $lName"), "\n\n";
 				}
 			}
 			print HEADER "\treturn strucRef;\n";
@@ -2205,7 +2226,11 @@ CONTAINED_STRUCTURE
 				my $lNameLen = length $lName;
 				my $lType = @{$structs{$_}[TYPES]}[$k];
 				my $inter = type2sv( $lType, "strucRef-> $lName");
-				print HEADER "\t(void) hv_store( $incHV, \"$lName\", $lNameLen, $inter, 0);\n";
+				my $prefix =
+					($structs{$_}->[DEFS]->[$k] =~ /^undef:/) ?
+						"if (!strucRef-> undef.$lName)" :
+						"(void)";
+				print HEADER "\t$prefix hv_store( $incHV, \"$lName\", $lNameLen, $inter, 0);\n";
 			}
 			print HEADER "\treturn newRV_noinc(( SV*) $incHV);\n";
 			print HEADER "}\n\n";
@@ -2264,7 +2289,7 @@ XS( $incBuild$ownCType)
 	dXSARGS;
 	$ownCType *$incRes;
 	char *$incClass;
-	
+
 	if ( items != 1)
 		croak ("Invalid usage of $ownOClass\:\:\%s", "$incBuild$ownCType");
 	{
@@ -2273,7 +2298,7 @@ XS( $incBuild$ownCType)
 		$incClass = HvNAME( SvSTASH( SvRV( ST(0))));
 		$incRes-> self = ( P${ownCType}_vmt) $incGetVmt( $incClass, ( PVMT)C${ownCType}, sizeof( ${ownCType}Vmt));
 		$incRes-> $hSuper = $incRes-> self-> super;
-		
+
 		ST( 0) = sv_newmortal();
 		sv_setiv( ST(0), (IV)( Handle)$incRes);
 	}
