@@ -107,7 +107,7 @@ sub Printers_Change
 {
 	my ( $self, $combic) = @_;
 	$::application-> get_printer-> printer( $combic-> text);
-	$self-> Device-> text( 'Device: ' . 
+	$self-> Device-> text( 'Device: ' .
 		$self-> {list}-> [ $combic-> List-> focusedItem]-> {device}
 	);
 }
@@ -136,8 +136,6 @@ sub execute
 
 1;
 
-__DATA__
-
 =pod
 
 =head1 NAME
@@ -160,9 +158,9 @@ selection process.
 
 =head1 SYNOPSIS
 
-	use Prima::PrintDialog;
+	use Prima qw(PrintDialog Application);
 
-	$dlg = Prima::PrintSetupDialog-> create;
+	my $dlg = Prima::PrintSetupDialog-> new;
 	if ( $dlg-> execute) {
 		my $p = $::application-> get_printer;
 		if ( $p-> begin_doc ) {
@@ -172,6 +170,10 @@ selection process.
 	}
 	$dlg-> destroy;
 
+=for podview <img src="printdialog.gif" cut=1>
+
+=for html <p><img src="https://raw.githubusercontent.com/dk/Prima/master/pod/Prima/printdialog.gif">
+
 =head1 AUTHOR
 
 Dmitry Karasik, E<lt>dmitry@karasik.eu.orgE<gt>.
